@@ -4,61 +4,74 @@ import java.io.IOException;
 import java.util.Collection;
 
 import com.fasterxml.jackson.annotation.JsonTypeInfo;
+
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.cfg.MapperConfig;
 import com.fasterxml.jackson.databind.jsontype.NamedType;
+import com.fasterxml.jackson.databind.jsontype.PolymorphicTypeValidator;
 import com.fasterxml.jackson.databind.jsontype.TypeIdResolver;
 import com.fasterxml.jackson.databind.jsontype.impl.ClassNameIdResolver;
 import com.fasterxml.jackson.databind.jsontype.impl.MinimalClassNameIdResolver;
 import com.fasterxml.jackson.databind.jsontype.impl.StdTypeResolverBuilder;
 import com.fasterxml.jackson.databind.type.TypeFactory;
+
 import com.fasterxml.jackson.dataformat.xml.util.StaxUtil;
 
 /**
  * Custom specialization of {@link StdTypeResolverBuilder}; needed so that
  * type id property name can be modified as necessary to make it legal
  * XML element or attribute name.
+ *<p>
+ * NOTE: Since 2.17, property name cleansing only applied to default
+ * names (like {@code "@class"} and {@code "@type"}) but not to explicitly
+ * specified ones (where caller presumably knows what to do).
  */
 public class XmlTypeResolverBuilder extends StdTypeResolverBuilder
 {
-    @Override
-    public StdTypeResolverBuilder init(JsonTypeInfo.Id idType, TypeIdResolver idRes)
-    {
-        super.init(idType, idRes);
-        if (_typeProperty != null) {
-            _typeProperty = StaxUtil.sanitizeXmlTypeName(_typeProperty);
-        }
-        return this;
+    public XmlTypeResolverBuilder() {
+    }
+
+    public XmlTypeResolverBuilder(JsonTypeInfo.Value settings) {
+        super(settings);
     }
 
     @Override
-    public StdTypeResolverBuilder typeProperty(String typeIdPropName)
-    {
-        // ok to have null/empty; will restore to use defaults
-        if (typeIdPropName == null || typeIdPropName.length() == 0) {
-            typeIdPropName = _idType.getDefaultPropertyName();
+    protected String _propName(String propName, JsonTypeInfo.Id idType) {
+        // 30-Jan-2024, tatu: Before 2.17 we used to indiscriminately cleanse
+        //   property name always; with 2.17+ only default ones
+        if (propName == null || propName.isEmpty()) {
+            propName = StaxUtil.sanitizeXmlTypeName(idType.getDefaultPropertyName());
+        } else {
+            // ... alas, there's... a "feature" (read: bug) in `JsonTypeInfo.Value` construction
+            // which will automatically apply default property name if no explicit property
+            // name specific. This means we don't really know if default is being used.
+            // But let's assume that if "propName.equals(defaultPropName)" this is the case.
+            if (propName.equals(idType.getDefaultPropertyName())) {
+                propName = StaxUtil.sanitizeXmlTypeName(propName);
+            }
         }
-        _typeProperty = StaxUtil.sanitizeXmlTypeName(typeIdPropName);
-        return this;
+        return propName;
     }
 
     @Override
     protected TypeIdResolver idResolver(MapperConfig<?> config,
-            JavaType baseType, Collection<NamedType> subtypes,
-            boolean forSer, boolean forDeser)
+            JavaType baseType, PolymorphicTypeValidator subtypeValidator,
+            Collection<NamedType> subtypes, boolean forSer, boolean forDeser)
     {
         if (_customIdResolver != null) {
             return _customIdResolver;
         }
-        // Only override handlings of class, minimal class; name is good as is
+        // Only override handlers of class, minimal class; name is good as is
         switch (_idType) {
         case CLASS:
-            return new XmlClassNameIdResolver(baseType, config.getTypeFactory());
+            return new XmlClassNameIdResolver(baseType, config.getTypeFactory(),
+                    subtypes, subTypeValidator(config));
         case MINIMAL_CLASS:
-            return new XmlMinimalClassNameIdResolver(baseType, config.getTypeFactory());
+            return new XmlMinimalClassNameIdResolver(baseType, config.getTypeFactory(),
+                    subtypes, subTypeValidator(config));
         default:
-            return super.idResolver(config, baseType, subtypes, forSer, forDeser);
         }
+        return super.idResolver(config, baseType, subtypeValidator, subtypes, forSer, forDeser);
     }
     
     /*
@@ -67,7 +80,6 @@ public class XmlTypeResolverBuilder extends StdTypeResolverBuilder
     /**********************************************************************
      */
 
-
     /**
      * Helper method for encoding regular Java class name in form that
      * can be used as XML element name.
@@ -116,9 +128,12 @@ public class XmlTypeResolverBuilder extends StdTypeResolverBuilder
     protected static class XmlClassNameIdResolver
         extends ClassNameIdResolver
     {
-        public XmlClassNameIdResolver(JavaType baseType, TypeFactory typeFactory)
+        private static final long serialVersionUID = 2L;
+
+        public XmlClassNameIdResolver(JavaType baseType, TypeFactory typeFactory,
+                Collection<NamedType> subtypes, PolymorphicTypeValidator ptv)
         {
-            super(baseType, typeFactory);
+            super(baseType, typeFactory, subtypes, ptv);
         }
 
         @Override
@@ -136,9 +151,12 @@ public class XmlTypeResolverBuilder extends StdTypeResolverBuilder
     protected static class XmlMinimalClassNameIdResolver
         extends MinimalClassNameIdResolver
     {
-        public XmlMinimalClassNameIdResolver(JavaType baseType, TypeFactory typeFactory)
+        private static final long serialVersionUID = 2L;
+
+        public XmlMinimalClassNameIdResolver(JavaType baseType, TypeFactory typeFactory,
+                Collection<NamedType> subtypes, PolymorphicTypeValidator ptv)
         {
-            super(baseType, typeFactory);
+            super(baseType, typeFactory, subtypes, ptv);
         }
 
         @Override
