@@ -1,17 +1,21 @@
 package com.fasterxml.jackson.dataformat.xml.deser;
 
 import java.io.IOException;
+import java.io.UncheckedIOException;
 import java.io.Writer;
 import java.math.BigDecimal;
 import java.math.BigInteger;
+import java.util.Objects;
 import java.util.Set;
 
+import javax.xml.XMLConstants;
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.base.ParserMinimalBase;
+import com.fasterxml.jackson.core.exc.StreamConstraintsException;
 import com.fasterxml.jackson.core.io.IOContext;
 import com.fasterxml.jackson.core.io.NumberInput;
 import com.fasterxml.jackson.core.util.ByteArrayBuilder;
@@ -19,6 +23,7 @@ import com.fasterxml.jackson.core.util.JacksonFeatureSet;
 
 import com.fasterxml.jackson.dataformat.xml.PackageVersion;
 import com.fasterxml.jackson.dataformat.xml.XmlMapper;
+import com.fasterxml.jackson.dataformat.xml.XmlNameProcessor;
 import com.fasterxml.jackson.dataformat.xml.util.CaseInsensitiveNameSet;
 import com.fasterxml.jackson.dataformat.xml.util.StaxUtil;
 
@@ -28,6 +33,7 @@ import com.fasterxml.jackson.dataformat.xml.util.StaxUtil;
  */
 public class FromXmlParser
     extends ParserMinimalBase
+    implements ElementWrappable // @since 2.15
 {
     /**
      * The default name placeholder for XML text segments is empty
@@ -53,6 +59,23 @@ public class FromXmlParser
      */
     public enum Feature implements FormatFeature
     {
+        /**
+         * Feature that enables automatic conversion of incoming "xsi:type"
+         * (where "type"  is the local name and "xsi" prefix is bound to URI
+         * {@link XMLConstants#W3C_XML_SCHEMA_INSTANCE_NS_URI}),
+         * into Jackson simple Property Name of {@code "xsi:type"}.
+         * This is usually needed to read content written using
+         * matching {@code ToXmlGenerator.Feature#AUTO_DETECT_XSI_TYPE} feature,
+         * usually used for Polymorphic handling where it is difficult
+         * to specify proper XML Namespace for type identifier.
+         * 
+         * Default setting is {@code false} in Jackson 2.x (for backwards compatibility):
+         * will be changed to true in Jackson 3.x
+         *
+         * @since 2.17
+         */
+        AUTO_DETECT_XSI_TYPE(false),
+
         /**
          * Feature that indicates whether XML Empty elements (ones where there are
          * no separate start and end tags, but just one tag that ends with "/&gt;")
@@ -60,7 +83,7 @@ public class FromXmlParser
          * returned as `null` tokens, they will be returned as {@link JsonToken#VALUE_STRING}
          * tokens with textual value of "" (empty String).
          *<p>
-         * Default setting was {@code true} (for backwards compatibility from 2.9 to 2.11 (inclusive)
+         * Default setting was {@code true} (for backwards compatibility) from 2.9 to 2.11 (inclusive)
          * but was changed in 2.12 to be {@code false} (see [dataformat-xml#411] for details)
          *
          * @since 2.9
@@ -161,6 +184,8 @@ public class FromXmlParser
     /**********************************************************
      */
 
+    protected final IOContext _ioContext;
+
     /**
      * Flag that indicates whether parser is closed or not. Gets
      * set when parser is either closed by explicit call
@@ -168,8 +193,6 @@ public class FromXmlParser
      */
     protected boolean _closed;
 
-    protected final IOContext _ioContext;
-
     /*
     /**********************************************************
     /* Parsing state
@@ -231,7 +254,7 @@ public class FromXmlParser
      */
 
     /**
-     * Bitfield that indicates which numeric representations
+     * Bit field that indicates which numeric representations
      * have been calculated for the current type
      */
     protected int _numTypesValid = NR_UNKNOWN;
@@ -252,17 +275,29 @@ public class FromXmlParser
      */
 
     public FromXmlParser(IOContext ctxt, int genericParserFeatures, int xmlFeatures,
-            ObjectCodec codec, XMLStreamReader xmlReader)
-        throws IOException
-    {
-        super(genericParserFeatures);
-        _formatFeatures = xmlFeatures;
+             ObjectCodec codec, XMLStreamReader xmlReader, XmlNameProcessor tagProcessor) throws IOException {
+        this(ctxt, genericParserFeatures, codec, new XmlTokenStream(xmlReader, ctxt.contentReference(), xmlFeatures, tagProcessor));
+    }
+
+    /**
+     * Constructs a new {@link FromXmlParser} instance using the provided XML token stream.
+     * This constructor initializes the parser with the given I/O context, parser features,
+     * and object codec for deserializing XML content into Java objects.
+     *
+     * @since 2.20
+     * @param ctxt I/O context used for handling low-level I/O operations and buffering
+     * @param genericParserFeatures set of bitmasked parser features to control parsing behavior
+     * @param codec object codec used for converting between JSON-like structures and Java objects
+     * @param xmlTokenStream the pre-processed XML token stream to parse from
+     * @throws IOException if an I/O error occurs during initialization or parsing setup
+     */
+    public FromXmlParser(IOContext ctxt, int genericParserFeatures, ObjectCodec codec, XmlTokenStream xmlTokenStream) throws IOException {
+        super(genericParserFeatures, ctxt.streamReadConstraints());
         _ioContext = ctxt;
         _objectCodec = codec;
         _parsingContext = XmlReadContext.createRootContext(-1, -1);
-        _xmlTokens = new XmlTokenStream(xmlReader, ctxt.contentReference(),
-                    _formatFeatures);
-
+        _xmlTokens = Objects.requireNonNull(xmlTokenStream, "xmlTokenStream cannot be null");
+        _formatFeatures = xmlTokenStream.getFormatFeatures();
         final int firstToken;
         try {
             firstToken = _xmlTokens.initialize();
@@ -275,10 +310,13 @@ public class FromXmlParser
         //    changed in 2.10.2
         if (_xmlTokens.hasXsiNil()) {
             _nextToken = JsonToken.VALUE_NULL;
+            // 21-Apr-2025, tatu: [dataformat-xml#714] Must "flush" the stream
+            _xmlTokens.markAsStreamEnd();
         } else {
             switch (firstToken) {
             case XmlTokenStream.XML_START_ELEMENT:
-            case XmlTokenStream.XML_DELAYED_START_ELEMENT:
+            // Removed from 2.14:
+            // case XmlTokenStream.XML_DELAYED_START_ELEMENT:
                 _nextToken = JsonToken.START_OBJECT;
                 break;
             case XmlTokenStream.XML_ROOT_TEXT:
@@ -378,7 +416,7 @@ public class FromXmlParser
         return this;
     }
 
-    /*                                                                                       
+    /*
     /**********************************************************                              
     /* FormatFeature support                                                                             
     /**********************************************************                              
@@ -418,27 +456,11 @@ public class FromXmlParser
 
     /*
     /**********************************************************
-    /* Internal API
+    /* ElementWrappable implementation
     /**********************************************************
      */
 
-    /**
-     * Method that may be called to indicate that specified names
-     * (only local parts retained currently: this may be changed in
-     * future) should be considered "auto-wrapping", meaning that
-     * they will be doubled to contain two opening elements, two
-     * matching closing elements. This is needed for supporting
-     * handling of so-called "unwrapped" array types, something
-     * XML mappings like JAXB often use.
-     *<p>
-     * NOTE: this method is considered part of internal implementation
-     * interface, and it is <b>NOT</b> guaranteed to remain unchanged
-     * between minor versions (it is however expected not to change in
-     * patch versions). So if you have to use it, be prepared for
-     * possible additional work.
-     * 
-     * @since 2.12
-     */
+    @Override
     public void addVirtualWrapping(Set<String> namesToWrap0, boolean caseInsensitive)
     {
 //System.out.printf("addVirtualWrapping(%s) at '%s' [case-insensitive? %s]\n", namesToWrap0, _parsingContext.pathAsPointer(), caseInsensitive);
@@ -462,11 +484,6 @@ public class FromXmlParser
         _parsingContext.setNamesToWrap(namesToWrap);
     }
 
-    @Deprecated // since 2.12
-    public void addVirtualWrapping(Set<String> namesToWrap) {
-        addVirtualWrapping(namesToWrap, false);
-    }
-
     /*
     /**********************************************************
     /* JsonParser impl
@@ -478,7 +495,7 @@ public class FromXmlParser
      * the current event.
      */
     @Override
-    public String getCurrentName() throws IOException
+    public String currentName() throws IOException
     {
         // start markers require information from parent
         String name;
@@ -495,6 +512,12 @@ public class FromXmlParser
         return name;
     }
 
+    @Deprecated // since 2.17
+    @Override
+    public String getCurrentName() throws IOException {
+        return currentName();
+    }
+
     @Override
     public void overrideCurrentName(String name)
     {
@@ -505,12 +528,16 @@ public class FromXmlParser
         }
         ctxt.setCurrentName(name);
     }
-    
+
     @Override
     public void close() throws IOException
     {
         if (!_closed) {
             _closed = true;
+            // 30-May-2025, tatu: was missing before 2.20
+            if (JsonParser.Feature.CLEAR_CURRENT_TOKEN_ON_CLOSE.enabledIn(_features)) {
+                _currToken = null;
+            }
             try {
                 if (_ioContext.isResourceManaged() || isEnabled(JsonParser.Feature.AUTO_CLOSE_SOURCE)) {
                     _xmlTokens.closeCompletely();
@@ -523,6 +550,7 @@ public class FromXmlParser
                 // Also, internal buffer(s) can now be released as well
                 _releaseBuffers();
             }
+            _ioContext.close();
         }
     }
 
@@ -540,7 +568,7 @@ public class FromXmlParser
      * that starts the current token.
      */
     @Override
-    public JsonLocation getTokenLocation() {
+    public JsonLocation currentTokenLocation() {
         return _xmlTokens.getTokenLocation();
     }
 
@@ -549,23 +577,41 @@ public class FromXmlParser
      * usually for error reporting purposes
      */
     @Override
-    public JsonLocation getCurrentLocation() {
+    public JsonLocation currentLocation() {
         return _xmlTokens.getCurrentLocation();
     }
 
+    @Deprecated // since 2.17
+    @Override
+    public JsonLocation getCurrentLocation() {
+        return currentLocation();
+    }
+
+    @Deprecated // since 2.17
+    @Override
+    public JsonLocation getTokenLocation() {
+        return currentTokenLocation();
+    }
+
     /**
      * Since xml representation can not really distinguish between array
      * and object starts (both are represented with elements), this method
-     * is overridden and taken to mean that expecation is that the current
+     * is overridden and taken to mean that expectation is that the current
      * start element is to mean 'start array', instead of default of
      * 'start object'.
+     *
+     * @throws UncheckedIOException if underlying {@link StreamReadConstraints} constraint fails
      */
     @Override
     public boolean isExpectedStartArrayToken()
     {
         JsonToken t = _currToken;
         if (t == JsonToken.START_OBJECT) {
-            _currToken = JsonToken.START_ARRAY;
+            try {
+                _updateToken(JsonToken.START_ARRAY);
+            } catch (StreamConstraintsException e) {
+                throw new UncheckedIOException(e);
+            }
             // Ok: must replace current context with array as well
             _parsingContext.convertToArray();
 //System.out.println(" FromXmlParser.isExpectedArrayStart(): OBJ->Array");
@@ -589,6 +635,8 @@ public class FromXmlParser
      * scalar types (numbers, booleans) -- they are all just Character Data,
      * without schema -- we can try to infer type from intent here.
      * The main benefit is avoiding checks for coercion.
+     *
+     * @throws UncheckedIOException if underlying {@link StreamReadConstraints} constraint fails
      */
     @Override
     public boolean isExpectedNumberIntToken()
@@ -597,50 +645,57 @@ public class FromXmlParser
         if (t == JsonToken.VALUE_STRING) {
             final String text = _currText.trim();
             final int len = _isIntNumber(text);
-            if (len > 0) {
-                if (len <= 9) {
-                    _numberInt = NumberInput.parseInt(text);
-                    _numTypesValid = NR_INT;
-                    _currToken = JsonToken.VALUE_NUMBER_INT;
-                    return true;
-                }
-                if (len <= 18) { // definitely in long range
-                    long l = NumberInput.parseLong(text);
-                    if (len == 10) {
-                        int asInt = (int) l;
-                        long l2 = (long) asInt;
-                        if (l == l2) {
-                            _numberInt = asInt;
-                            _numTypesValid = NR_INT;
-                            _currToken = JsonToken.VALUE_NUMBER_INT;
+            try {
+                if (len > 0) {
+                    if (len <= 9) {
+                        _numberInt = NumberInput.parseInt(text);
+                        _numTypesValid = NR_INT;
+                        _updateToken(JsonToken.VALUE_NUMBER_INT);
+                        return true;
+                    }
+                    if (len <= 18) { // definitely in long range
+                        long l = NumberInput.parseLong(text);
+                        if (len == 10) {
+                            int asInt = (int) l;
+                            long l2 = (long) asInt;
+                            if (l == l2) {
+                                _numberInt = asInt;
+                                _numTypesValid = NR_INT;
+                                _updateToken(JsonToken.VALUE_NUMBER_INT);
+                                return true;
+                            }
+                        }
+                        _numberLong = l;
+                        _numTypesValid = NR_LONG;
+                        _updateToken(JsonToken.VALUE_NUMBER_INT);
+                        return true;
+                    }
+                    // Might still fit within `long`
+                    if (len == 19) {
+                        final boolean stillLong;
+                        if (text.charAt(0) == '-') {
+                            stillLong = NumberInput.inLongRange(text.substring(1), true);
+                        } else {
+                            stillLong = NumberInput.inLongRange(text, false);
+                        }
+                        if (stillLong) {
+                            _numberLong = NumberInput.parseLong(text);
+                            _numTypesValid = NR_LONG;
+                            _updateToken(JsonToken.VALUE_NUMBER_INT);
                             return true;
                         }
                     }
-                    _numberLong = l;
-                    _numTypesValid = NR_LONG;
-                    _currToken = JsonToken.VALUE_NUMBER_INT;
+                    // finally, need BigInteger
+                    streamReadConstraints().validateIntegerLength(text.length());
+                    _numberBigInt = NumberInput.parseBigInteger(
+                        text, isEnabled(StreamReadFeature.USE_FAST_BIG_NUMBER_PARSER));
+                    _numTypesValid = NR_BIGINT;
+                    _updateToken(JsonToken.VALUE_NUMBER_INT);
                     return true;
                 }
-                // Might still fit within `long`
-                if (len == 19) {
-                    final boolean stillLong;
-                    if (text.charAt(0) == '-') {
-                        stillLong = NumberInput.inLongRange(text.substring(1), true);
-                    } else {
-                        stillLong = NumberInput.inLongRange(text, false);
-                    }
-                    if (stillLong) {
-                        _numberLong = NumberInput.parseLong(text);
-                        _numTypesValid = NR_LONG;
-                        _currToken = JsonToken.VALUE_NUMBER_INT;
-                        return true;
-                    }
-                }
-                // finally, need BigInteger
-                _numberBigInt = new BigInteger(text);
-                _numTypesValid = NR_BIGINT;
-                _currToken = JsonToken.VALUE_NUMBER_INT;
-                return true;
+            } catch (StreamConstraintsException e) {
+                // Ugh. This method in API ought to expose IOException
+                throw new UncheckedIOException(e);
             }
         }
         return (t == JsonToken.VALUE_NUMBER_INT);
@@ -653,16 +708,16 @@ public class FromXmlParser
     {
         JsonToken t = nextToken0();
         if (t != null) {
-            final String loc = (_parsingContext == null) ? "NULL" : String.valueOf(_parsingContext.pathAsPointer());
+            final String loc = (_parsingContext == null) ? "<null>" : "'"+String.valueOf(_parsingContext.pathAsPointer())+"'";
             switch (t) {
             case FIELD_NAME:
-                System.out.printf("FromXmlParser.nextToken() at '%s': JsonToken.FIELD_NAME '%s'\n", loc, _parsingContext.getCurrentName());
+                System.out.printf("FromXmlParser.nextToken() at %s: JsonToken.FIELD_NAME '%s'\n", loc, _parsingContext.getCurrentName());
                 break;
             case VALUE_STRING:
-                System.out.printf("FromXmlParser.nextToken() at '%s': JsonToken.VALUE_STRING '%s'\n", loc, getText());
+                System.out.printf("FromXmlParser.nextToken() at %s: JsonToken.VALUE_STRING '%s'\n", loc, getText());
                 break;
             default:
-                System.out.printf("FromXmlParser.nextToken() at '%s': %s\n", loc, t);
+                System.out.printf("FromXmlParser.nextToken() at %s: %s\n", loc, t);
             }
         }
         return t;
@@ -677,8 +732,7 @@ public class FromXmlParser
         _numTypesValid = NR_UNKNOWN;
 //System.out.println("FromXmlParser.nextToken0: _nextToken = "+_nextToken);
         if (_nextToken != null) {
-            JsonToken t = _nextToken;
-            _currToken = t;
+            final JsonToken t = _updateToken(_nextToken);
             _nextToken = null;
 
             switch (t) {
@@ -720,7 +774,7 @@ public class FromXmlParser
                 // leave _mayBeLeaf set, as we start a new context
                 _nextToken = JsonToken.FIELD_NAME;
                 _parsingContext = _parsingContext.createChildObjectContext(-1, -1);
-                return (_currToken = JsonToken.START_OBJECT);
+                return _updateToken(JsonToken.START_OBJECT);
             }
             if (_parsingContext.inArray()) {
                 // Yup: in array, so this element could be verified; but it won't be
@@ -741,7 +795,7 @@ public class FromXmlParser
             _mayBeLeaf = true;
             // Ok: in array context we need to skip reporting field names.
             // But what's the best way to find next token?
-            return (_currToken = JsonToken.FIELD_NAME);
+            return _updateToken(JsonToken.FIELD_NAME);
         }
 
         // Ok; beyond start element, what do we get?
@@ -756,16 +810,16 @@ public class FromXmlParser
                         //    expose as empty Object, not null
                         _nextToken = JsonToken.END_OBJECT;
                         _parsingContext = _parsingContext.createChildObjectContext(-1, -1);
-                        return (_currToken = JsonToken.START_OBJECT);
+                        return _updateToken(JsonToken.START_OBJECT);
                     }
                     // 07-Sep-2019, tatu: for [dataformat-xml#353], must NOT return second null
                     if (_currToken != JsonToken.VALUE_NULL) {
                         // 13-May-2020, tatu: [dataformat-xml#397]: advance `index`
                         _parsingContext.valueStarted();
-                        return (_currToken = JsonToken.VALUE_NULL);
+                        return _updateToken(JsonToken.VALUE_NULL);
                     }
                 }
-                _currToken = _parsingContext.inArray() ? JsonToken.END_ARRAY : JsonToken.END_OBJECT;
+                _updateToken(_parsingContext.inArray() ? JsonToken.END_ARRAY : JsonToken.END_OBJECT);
                 _parsingContext = _parsingContext.getParent();
                 return _currToken;
 
@@ -776,15 +830,15 @@ public class FromXmlParser
                     _nextToken = JsonToken.FIELD_NAME;
                     _currText = _xmlTokens.getText();
                     _parsingContext = _parsingContext.createChildObjectContext(-1, -1);
-                    return (_currToken = JsonToken.START_OBJECT);
+                    return _updateToken(JsonToken.START_OBJECT);
                 }
                 _parsingContext.setCurrentName(_xmlTokens.getLocalName());
-                return (_currToken = JsonToken.FIELD_NAME);
+                return _updateToken(JsonToken.FIELD_NAME);
             case XmlTokenStream.XML_ATTRIBUTE_VALUE:
                 _currText = _xmlTokens.getText();
                 // 13-May-2020, tatu: [dataformat-xml#397]: advance `index`
                 _parsingContext.valueStarted();
-                return (_currToken = JsonToken.VALUE_STRING);
+                return _updateToken(JsonToken.VALUE_STRING);
             case XmlTokenStream.XML_TEXT:
                 _currText = _xmlTokens.getText();
                 if (_mayBeLeaf) {
@@ -806,10 +860,10 @@ public class FromXmlParser
                                 //    be done, by swallowing the token)
                                 _nextToken = JsonToken.END_OBJECT;
                                 _parsingContext = _parsingContext.createChildObjectContext(-1, -1);
-                                return (_currToken = JsonToken.START_OBJECT);
+                                return _updateToken(JsonToken.START_OBJECT);
                             }
                         }
-                        return (_currToken = JsonToken.VALUE_STRING);
+                        return _updateToken(JsonToken.VALUE_STRING);
                     }
                     if (token != XmlTokenStream.XML_START_ELEMENT) {
                         throw new JsonParseException(this, String.format(
@@ -831,7 +885,7 @@ XmlTokenStream.XML_END_ELEMENT, XmlTokenStream.XML_START_ELEMENT, token));
                         //    along is not enough.
                         _nextIsLeadingMixed = true;
                         _nextToken = JsonToken.FIELD_NAME;
-                        return (_currToken = JsonToken.START_OBJECT);
+                        return _updateToken(JsonToken.START_OBJECT);
                     } else if (XmlTokenStream._allWs(_currText)) {
                         token = _nextToken();
                         continue;
@@ -844,17 +898,25 @@ XmlTokenStream.XML_END_ELEMENT, XmlTokenStream.XML_START_ELEMENT, token));
                     }
                     // 29-Mar-2021, tatu: This seems like an error condition...
                     //   How should we indicate it? As of 2.13, report as unexpected state
+                    /*
                     throw _constructError(
-"Unexpected non-whitespace text ('"+_currText+"' in Array context: should not occur (or should be handled)"
+"Unexpected non-whitespace text ('"+_currText+"') in Array context: should not occur (or should be handled)"
 );
+                    */
+                    
+                    // [dataformat-xml#509] 2.13 introduced a defect in which an Exception was thrown above, breaking
+                    // parsing of mixed content arrays (https://github.com/FasterXML/jackson-dataformat-xml/issues/509).
+                    // This exception case was removed to enable continued support of that functionality, but more
+                    // robust state handling may be in order.
+                    // See comment https://github.com/FasterXML/jackson-dataformat-xml/pull/604
                 }
 
                 // If not a leaf (or otherwise ignorable), need to transform into property...
                 _parsingContext.setCurrentName(_cfgNameForTextElement);
                 _nextToken = JsonToken.VALUE_STRING;
-                return (_currToken = JsonToken.FIELD_NAME);
+                return _updateToken(JsonToken.FIELD_NAME);
             case XmlTokenStream.XML_END:
-                return (_currToken = null);
+                return _updateTokenToNull();
             default:
                 return _internalErrorUnknownToken(token);
             }
@@ -871,7 +933,7 @@ XmlTokenStream.XML_END_ELEMENT, XmlTokenStream.XML_START_ELEMENT, token));
     @Override
     public String nextFieldName() throws IOException {
         if (nextToken() == JsonToken.FIELD_NAME) {
-            return getCurrentName();
+            return currentName();
         }
         return null;
     }
@@ -886,8 +948,7 @@ XmlTokenStream.XML_END_ELEMENT, XmlTokenStream.XML_START_ELEMENT, token));
     {
         _binaryValue = null;
         if (_nextToken != null) {
-            JsonToken t = _nextToken;
-            _currToken = t;
+            final JsonToken t = _updateToken(_nextToken);
             _nextToken = null;
 
             // expected case; yes, got a String
@@ -907,7 +968,7 @@ XmlTokenStream.XML_END_ELEMENT, XmlTokenStream.XML_START_ELEMENT, token));
             if (_mayBeLeaf) {
                 _nextToken = JsonToken.FIELD_NAME;
                 _parsingContext = _parsingContext.createChildObjectContext(-1, -1);
-                _currToken = JsonToken.START_OBJECT;
+                _updateToken(JsonToken.START_OBJECT);
                 return null;
             }
             if (_parsingContext.inArray()) {
@@ -922,7 +983,7 @@ XmlTokenStream.XML_END_ELEMENT, XmlTokenStream.XML_START_ELEMENT, token));
                 _xmlTokens.repeatStartElement();
             }
             _mayBeLeaf = true;
-            _currToken = JsonToken.FIELD_NAME;
+            _updateToken(JsonToken.FIELD_NAME);
             return null;
         }
 
@@ -930,14 +991,18 @@ XmlTokenStream.XML_END_ELEMENT, XmlTokenStream.XML_START_ELEMENT, token));
         switch (token) {
         case XmlTokenStream.XML_END_ELEMENT:
             if (_mayBeLeaf) {
-                // NOTE: this is different from nextToken() -- produce "", NOT null
                 _mayBeLeaf = false;
-                _currToken = JsonToken.VALUE_STRING;
+                // 18-Mar-2023, tatu: [dataformat-xml#584 / #585] in 2.14 and before
+                //    returned VALUE_STRING on assumption we never expose `null`s if
+                //    asked text value -- but that seems incorrect. Hoping this won't
+                //    break anything in 2.15+
+
+                _updateToken(JsonToken.VALUE_NULL);
                 // 13-May-2020, tatu: [dataformat-xml#397]: advance `index`
                 _parsingContext.valueStarted();
-                return (_currText = "");
+                return (_currText = null);
             }
-            _currToken = _parsingContext.inArray() ? JsonToken.END_ARRAY : JsonToken.END_OBJECT;
+            _updateToken(_parsingContext.inArray() ? JsonToken.END_ARRAY : JsonToken.END_OBJECT);
             _parsingContext = _parsingContext.getParent();
             break;
         case XmlTokenStream.XML_ATTRIBUTE_NAME:
@@ -947,14 +1012,14 @@ XmlTokenStream.XML_END_ELEMENT, XmlTokenStream.XML_START_ELEMENT, token));
                 _nextToken = JsonToken.FIELD_NAME;
                 _currText = _xmlTokens.getText();
                 _parsingContext = _parsingContext.createChildObjectContext(-1, -1);
-                _currToken = JsonToken.START_OBJECT;
+                _updateToken(JsonToken.START_OBJECT);
             } else {
                 _parsingContext.setCurrentName(_xmlTokens.getLocalName());
-                _currToken = JsonToken.FIELD_NAME;
+                _updateToken(JsonToken.FIELD_NAME);
             }
             break;
         case XmlTokenStream.XML_ATTRIBUTE_VALUE:
-            _currToken = JsonToken.VALUE_STRING;
+            _updateToken(JsonToken.VALUE_STRING);
             // 13-May-2020, tatu: [dataformat-xml#397]: advance `index`
             _parsingContext.valueStarted();
             return (_currText = _xmlTokens.getText());
@@ -968,16 +1033,16 @@ XmlTokenStream.XML_END_ELEMENT, XmlTokenStream.XML_START_ELEMENT, token));
                 // for otherwise empty List/array
                 // 13-May-2020, tatu: [dataformat-xml#397]: advance `index`
                 _parsingContext.valueStarted();
-                _currToken = JsonToken.VALUE_STRING;
+                _updateToken(JsonToken.VALUE_STRING);
                 return _currText;
             }
             // If not a leaf, need to transform into property...
             _parsingContext.setCurrentName(_cfgNameForTextElement);
             _nextToken = JsonToken.VALUE_STRING;
-            _currToken = JsonToken.FIELD_NAME;
+            _updateToken(JsonToken.FIELD_NAME);
             break;
         case XmlTokenStream.XML_END:
-            _currToken = null;
+            _updateTokenToNull();
         default:
             return _internalErrorUnknownToken(token);
         }
@@ -1020,7 +1085,7 @@ XmlTokenStream.XML_END_ELEMENT, XmlTokenStream.XML_START_ELEMENT, token));
         }
         switch (_currToken) {
         case FIELD_NAME:
-            return getCurrentName();
+            return currentName();
         case VALUE_STRING:
             return _currText;
         default:
@@ -1134,6 +1199,15 @@ XmlTokenStream.XML_END_ELEMENT, XmlTokenStream.XML_START_ELEMENT, token));
         return NumberType.BIG_INTEGER;
     }
 
+    /**
+     * XML has no notion of natural/native floating-point type (has to be
+     * provided externally via Schema or so), so need to ensure we indicate that.
+     */
+    @Override // added in 2.17
+    public NumberTypeFP getNumberTypeFP() throws IOException {
+        return NumberTypeFP.UNKNOWN;
+    }
+
     @Override
     public Number getNumberValue() throws IOException {
         if (_numTypesValid == NR_UNKNOWN) {
