@@ -39,8 +39,14 @@ public class XmlBeanSerializer extends XmlBeanSerializerBase
         super(src, objectIdWriter);
     }
 
-    public XmlBeanSerializer(XmlBeanSerializerBase src, Set<String> toIgnore) {
-        super(src, toIgnore);
+    public XmlBeanSerializer(XmlBeanSerializerBase src, Set<String> toIgnore, Set<String> toInclude) {
+        super(src, toIgnore, toInclude);
+    }
+
+    protected XmlBeanSerializer(XmlBeanSerializerBase src,
+            BeanPropertyWriter[] properties, BeanPropertyWriter[] filteredProperties)
+    {
+        super(src, properties, filteredProperties);
     }
 
     /*
@@ -64,9 +70,15 @@ public class XmlBeanSerializer extends XmlBeanSerializerBase
         return new XmlBeanSerializer(this, _objectIdWriter, filterId);
     }
 
-    @Override
-    protected BeanSerializerBase withIgnorals(Set<String> toIgnore) {
-        return new XmlBeanSerializer(this, toIgnore);
+    @Override // since 2.12
+    protected BeanSerializerBase withByNameInclusion(Set<String> toIgnore, Set<String> toInclude) {
+        return new XmlBeanSerializer(this, toIgnore, toInclude);
+    }
+
+    @Override // since 2.11.1
+    protected BeanSerializerBase withProperties(BeanPropertyWriter[] properties,
+            BeanPropertyWriter[] filteredProperties) {
+        return new XmlBeanSerializer(this, properties, filteredProperties);
     }
 
     /**
@@ -82,10 +94,8 @@ public class XmlBeanSerializer extends XmlBeanSerializerBase
          * 
          * - have Object Id (may be allowed in future)
          * - have any getter
-         * 
          */
         if ((_objectIdWriter == null)
-                && (_anyGetterWriter == null)
                 && (_propertyFilterId == null)
                 ) {
             return new BeanAsArraySerializer(this);
@@ -105,19 +115,19 @@ public class XmlBeanSerializer extends XmlBeanSerializerBase
      * {@link BeanPropertyWriter} instances.
      */
     @Override
-    public void serialize(Object bean, JsonGenerator jgen, SerializerProvider provider) throws IOException
+    public void serialize(Object bean, JsonGenerator g, SerializerProvider provider) throws IOException
     {
         if (_objectIdWriter != null) {
-            _serializeWithObjectId(bean, jgen, provider, true);
+            _serializeWithObjectId(bean, g, provider, true);
             return;
         }
-        jgen.writeStartObject();
+        g.writeStartObject();
         if (_propertyFilterId != null) {
-            serializeFieldsFiltered(bean, jgen, provider);
+            serializeFieldsFiltered(bean, g, provider);
         } else {
-            serializeFields(bean, jgen, provider);
+            serializeFields(bean, g, provider);
         }
-        jgen.writeEndObject();
+        g.writeEndObject();
     }
 
     /*
