@@ -63,9 +63,8 @@ public abstract class XmlBeanSerializerBase extends BeanSerializerBase
     {
         super(src);
 
-        /* Then make sure attributes are sorted before elements, keep track
-         * of how many there are altogether
-         */
+        // Then make sure attributes are sorted before elements, keep track
+        // of how many there are altogether
         int attrCount = 0;
         for (BeanPropertyWriter bpw : _props) {
             if (_isAttribute(bpw)) { // Yup: let's build re-ordered list then
@@ -124,9 +123,10 @@ public abstract class XmlBeanSerializerBase extends BeanSerializerBase
         _cdata = src._cdata;
     }
 
-    protected XmlBeanSerializerBase(XmlBeanSerializerBase src, Set<String> toIgnore)
+    protected XmlBeanSerializerBase(XmlBeanSerializerBase src,
+            Set<String> toIgnore, Set<String> toInclude)
     {
-        super(src, toIgnore);
+        super(src, toIgnore, toInclude);
         _attributeCount = src._attributeCount;
         _textPropertyIndex = src._textPropertyIndex;
         _xmlNames = src._xmlNames;
@@ -142,6 +142,16 @@ public abstract class XmlBeanSerializerBase extends BeanSerializerBase
         _cdata = src._cdata;
     }
 
+    // @since 2.11.1
+    protected XmlBeanSerializerBase(XmlBeanSerializerBase src,
+            BeanPropertyWriter[] properties, BeanPropertyWriter[] filteredProperties) {
+        super(src, properties, filteredProperties);
+        _attributeCount = src._attributeCount;
+        _textPropertyIndex = src._textPropertyIndex;
+        _xmlNames = src._xmlNames;
+        _cdata = src._cdata;
+    }
+
     /*
     /**********************************************************
     /* Overridden serialization methods
@@ -162,7 +172,6 @@ public abstract class XmlBeanSerializerBase extends BeanSerializerBase
             super.serializeFields(bean, gen0, provider);
             return;
         }
-        
         final ToXmlGenerator xgen = (ToXmlGenerator) gen0;
         final BeanPropertyWriter[] props;
         if (_filteredProps != null && provider.getActiveView() != null) {
@@ -209,12 +218,6 @@ public abstract class XmlBeanSerializerBase extends BeanSerializerBase
                     xgen.setNextIsUnwrapped(false);
                 }
             }
-            if (_anyGetterWriter != null) {
-                // For [#117]: not a clean fix, but with @JsonTypeInfo, we'll end up
-                // with accidental attributes otherwise
-                xgen.setNextIsAttribute(false);
-                _anyGetterWriter.getAndSerialize(bean, xgen, provider);
-            }
         } catch (Exception e) {
             String name = (i == props.length) ? "[anySetter]" : props[i].getName();
             wrapAndThrow(provider, e, bean, name);
@@ -285,12 +288,11 @@ public abstract class XmlBeanSerializerBase extends BeanSerializerBase
                         filter.serializeAsField(bean, xgen, provider, prop);
                     }
                 }
-            }
-            if (_anyGetterWriter != null) {
-                // For [#117]: not a clean fix, but with @JsonTypeInfo, we'll end up
-                // with accidental attributes otherwise
-                xgen.setNextIsAttribute(false);
-                _anyGetterWriter.getAndSerialize(bean, xgen, provider);
+                // Reset to avoid next value being written as unwrapped,
+                // for example when property is suppressed
+                if (i == textIndex) {
+                    xgen.setNextIsUnwrapped(false);
+                }
             }
         } catch (Exception e) {
             String name = (i == props.length) ? "[anySetter]" : props[i].getName();
