@@ -1,17 +1,29 @@
 package com.fasterxml.jackson.dataformat.xml;
 
 import java.io.*;
+import java.nio.charset.StandardCharsets;
 import java.util.Arrays;
 
 import junit.framework.TestCase;
 
 import com.fasterxml.jackson.annotation.JsonPropertyOrder;
 import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.databind.AnnotationIntrospector;
+import com.fasterxml.jackson.databind.type.TypeFactory;
 import com.fasterxml.jackson.dataformat.xml.annotation.JacksonXmlProperty;
+import com.fasterxml.jackson.module.jakarta.xmlbind.JakartaXmlBindAnnotationIntrospector;
 
+@Deprecated // since 2.19
 public abstract class XmlTestBase
     extends TestCase
 {
+    protected static final String DEFAULT_NEW_LINE;
+
+    static {
+        String newLine = System.getProperty("line.separator");
+        DEFAULT_NEW_LINE = newLine == null ? "\n" : newLine;
+    }
+
     @JsonPropertyOrder({ "first", "last", "id" })
     protected static class NameBean {
         @JacksonXmlProperty(isAttribute=true)
@@ -102,7 +114,7 @@ public abstract class XmlTestBase
             return true;
         }
     }
-    
+
     protected static class StringBean
     {
         public String text;
@@ -116,7 +128,7 @@ public abstract class XmlTestBase
             return "\""+text+"\"";
         }
     }
-    
+
     /**
      * Simple wrapper around String type, usually to test value
      * conversions or wrapping
@@ -129,7 +141,7 @@ public abstract class XmlTestBase
             str = value;
         }
     }
-    
+
     protected static class IntWrapper {
         public int i;
 
@@ -138,6 +150,30 @@ public abstract class XmlTestBase
             i = value;
         }
     }
+
+    public static class Point {
+        public int x, y;
+
+        protected Point() { } // for deser
+        public Point(int x0, int y0) {
+            x = x0;
+            y = y0;
+        }
+    
+        @Override
+        public boolean equals(Object o) {
+            if (!(o instanceof Point)) {
+                return false;
+            }
+            Point other = (Point) o;
+            return (other.x == x) && (other.y == y);
+        }
+
+        @Override
+        public String toString() {
+            return String.format("[x=%d, y=%d]", x, y);
+        }
+    }
     
     /*
     /**********************************************************
@@ -171,7 +207,7 @@ public abstract class XmlTestBase
         +"  }"
         +"}"
         ;
-    
+
     /*
     /**********************************************************
     /* Construction, factory methods
@@ -182,13 +218,33 @@ public abstract class XmlTestBase
         super();
     }
 
+    protected XmlFactoryBuilder streamFactoryBuilder() {
+        return XmlFactory.builder();
+    }
+
+    protected static XmlMapper newMapper() {
+        return new XmlMapper();
+    }
+
+    protected static XmlMapper.Builder mapperBuilder() {
+        return XmlMapper.builder();
+    }
+
+    protected static XmlMapper.Builder mapperBuilder(XmlFactory f) {
+        return XmlMapper.builder(f);
+    }
+
     protected XmlMapper xmlMapper(boolean useListWrapping)
     {
         JacksonXmlModule module = new JacksonXmlModule();
         module.setDefaultUseWrapper(useListWrapping);
         return new XmlMapper(module);
     }
-    
+
+    protected AnnotationIntrospector jakartaXMLBindAnnotationIntrospector() {
+        return new JakartaXmlBindAnnotationIntrospector(TypeFactory.defaultInstance());
+    }
+
     /*
     /**********************************************************
     /* Additional assertion methods
@@ -233,7 +289,7 @@ public abstract class XmlTestBase
         throws IOException
     {
         assertEquals(expName, jp.getText());
-        assertEquals(expName, jp.getCurrentName());
+        assertEquals(expName, jp.currentName());
     }
 
     protected void verifyException(Throwable e, String... matches)
@@ -246,7 +302,8 @@ public abstract class XmlTestBase
                 return;
             }
         }
-        fail("Expected an exception with one of substrings ("+Arrays.asList(matches)+"): got one with message \""+msg+"\"");
+        fail("Expected an exception with one of substrings ("+Arrays.asList(matches)+"): got one ("+
+                e.getClass().getName()+") with message \""+msg+"\"");
     }
     
     /*
@@ -255,16 +312,12 @@ public abstract class XmlTestBase
     /**********************************************************
      */
 
-    protected static String aposToQuotes(String json) {
-        return json.replace("'", "\"");
+    protected static String a2q(String content) {
+        return content.replace("'", "\"");
     }
 
     protected byte[] utf8Bytes(String str) {
-        try {
-            return str.getBytes("UTF-8");
-        } catch (IOException e) {
-            throw new IllegalArgumentException(e);
-        }
+        return str.getBytes(StandardCharsets.UTF_8);
     }
     
     /**
@@ -295,13 +348,36 @@ public abstract class XmlTestBase
         return sb.toString();
     }
 
+    protected byte[] readResource(String ref)
+    {
+       ByteArrayOutputStream bytes = new ByteArrayOutputStream();
+       final byte[] buf = new byte[4000];
+
+       InputStream in = getClass().getResourceAsStream(ref);
+       if (in != null) {
+           try {
+               int len;
+               while ((len = in.read(buf)) > 0) {
+                   bytes.write(buf, 0, len);
+               }
+               in.close();
+           } catch (IOException e) {
+               throw new RuntimeException("Failed to read resource '"+ref+"': "+e);
+           }
+       }
+       if (bytes.size() == 0) {
+           throw new IllegalArgumentException("Failed to read resource '"+ref+"': empty resource?");
+       }
+       return bytes.toByteArray();
+    }
+    
     public String jaxbSerialized(Object ob, Class<?>... classes) throws Exception
     {
         StringWriter sw = new StringWriter();
         if (classes.length == 0) {
-            javax.xml.bind.JAXB.marshal(ob, sw);
+            jakarta.xml.bind.JAXB.marshal(ob, sw);
         } else {
-            javax.xml.bind.JAXBContext.newInstance(classes).createMarshaller().marshal(ob, sw);
+            jakarta.xml.bind.JAXBContext.newInstance(classes).createMarshaller().marshal(ob, sw);
         }
         sw.close();
         return sw.toString();
