@@ -1,16 +1,22 @@
 package com.fasterxml.jackson.dataformat.xml.lists;
 
-import java.util.*;
+import java.util.ArrayList;
+import java.util.List;
+
+import org.junit.jupiter.api.Test;
 
 import com.fasterxml.jackson.annotation.JsonPropertyOrder;
-
+import com.fasterxml.jackson.annotation.JsonRootName;
 import com.fasterxml.jackson.dataformat.xml.XmlMapper;
-import com.fasterxml.jackson.dataformat.xml.XmlTestBase;
+import com.fasterxml.jackson.dataformat.xml.XmlTestUtil;
 import com.fasterxml.jackson.dataformat.xml.annotation.*;
 
-public class ListRoundtripTest extends XmlTestBase
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+
+public class ListRoundtripTest extends XmlTestUtil
 {
-    @JacksonXmlRootElement(localName="parents")
+    @JsonRootName("parents")
     public static class Parents {
         @JacksonXmlElementWrapper(useWrapping=false)
         public List<Parent> parent = new ArrayList<Parent>();
@@ -46,8 +52,7 @@ public class ListRoundtripTest extends XmlTestBase
         }
     }
 
-    // For [Issue#58]
-    @JacksonXmlRootElement(localName = "point")
+    @JsonRootName("point")
     static class Point {
         @JacksonXmlProperty(localName = "x", isAttribute = true)
         int x;
@@ -58,7 +63,7 @@ public class ListRoundtripTest extends XmlTestBase
         public Point(int x, int y) { this.x = x; this.y = y; }
     }
 
-    @JacksonXmlRootElement(localName = "Points")
+    @JsonRootName("Points")
     static class PointContainer {
         @JacksonXmlElementWrapper(useWrapping = false)
         @JacksonXmlProperty(localName = "point")
@@ -90,6 +95,7 @@ public class ListRoundtripTest extends XmlTestBase
 //        MAPPER.enable(SerializationFeature.INDENT_OUTPUT);
     }
     
+    @Test
     public void testParentListRoundtrip() throws Exception
     {
         Parents root = new Parents();
@@ -120,6 +126,7 @@ public class ListRoundtripTest extends XmlTestBase
         assertEquals("2", prop2.value);
     }
 
+    @Test
     public void testListWithAttrOnlyValues() throws Exception
     {
         PointContainer obj = new PointContainer();
@@ -144,6 +151,7 @@ public class ListRoundtripTest extends XmlTestBase
 
     // // [Issue#64]
     
+    @Test
     public void testOptionals() throws Exception
     {
         Optionals ob = MAPPER.readValue("<MultiOptional><optional type='work'>123-456-7890</optional></MultiOptional>",
@@ -158,6 +166,7 @@ public class ListRoundtripTest extends XmlTestBase
     }
 
     /*// comment out for release
+    @Test
     public void testOptionalsWithMissingType() throws Exception
     {
 //        Optionals ob = MAPPER.readValue("<MultiOptional><optional type='work'>123-456-7890</optional></MultiOptional>",
