@@ -2,51 +2,143 @@ package com.fasterxml.jackson.dataformat.xml.stream;
 
 import java.io.*;
 
-import com.fasterxml.jackson.core.*;
+import org.junit.jupiter.api.Test;
 
+import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.JsonParser.NumberType;
 import com.fasterxml.jackson.databind.JsonNode;
 import com.fasterxml.jackson.databind.ObjectMapper;
-
-import com.fasterxml.jackson.dataformat.xml.XmlFactory;
-import com.fasterxml.jackson.dataformat.xml.XmlMapper;
-import com.fasterxml.jackson.dataformat.xml.XmlTestBase;
+import com.fasterxml.jackson.dataformat.xml.*;
 import com.fasterxml.jackson.dataformat.xml.deser.FromXmlParser;
 
-public class XmlParserTest extends XmlTestBase
-{
-    protected JsonFactory _jsonFactory;
-    protected XmlFactory _xmlFactory;
-    protected XmlMapper _xmlMapper;
+import static org.junit.jupiter.api.Assertions.*;
 
-    // let's actually reuse XmlMapper to make things bit faster
-    @Override
-    public void setUp() throws Exception {
-        super.setUp();
-        _jsonFactory = new JsonFactory();
-        _xmlFactory = new XmlFactory();
-        _xmlMapper = new XmlMapper();
+public class XmlParserTest extends XmlTestUtil
+{
+    protected final JsonFactory _jsonFactory = new JsonFactory();
+    protected final XmlMapper _xmlMapper = newMapper();
+    protected XmlFactory _xmlFactory = _xmlMapper.getFactory();
+
+    /*
+    /**********************************************************************
+    /* Unit tests, simplest/manual
+    /**********************************************************************
+     */
+    
+    @Test
+    public void testSimplest() throws Exception
+    {
+        final String XML = "<root><leaf>abc</leaf></root>";
+        // -> "{\"leaf\":\"abc\"}"
+
+        try (JsonParser p = _xmlMapper.createParser(XML)) {
+            assertEquals(0, p.getParsingContext().getNestingDepth());
+            assertToken(JsonToken.START_OBJECT, p.nextToken());
+            assertEquals(1, p.getParsingContext().getNestingDepth());
+            assertToken(JsonToken.FIELD_NAME, p.nextToken());
+            assertEquals("leaf", p.currentName());
+            assertEquals("leaf", p.getText());
+            assertEquals("leaf", p.getValueAsString());
+            assertEquals("leaf", p.getValueAsString("x"));
+            assertEquals(1, p.getParsingContext().getNestingDepth());
+            assertToken(JsonToken.VALUE_STRING, p.nextToken());
+            assertEquals("abc", p.getText());
+            assertEquals("abc", p.getText());
+            assertEquals("abc", p.getValueAsString());
+            assertEquals("abc", p.getValueAsString("x"));
+            assertEquals(1, p.getParsingContext().getNestingDepth());
+            assertToken(JsonToken.END_OBJECT, p.nextToken());
+            assertEquals(0, p.getParsingContext().getNestingDepth());
+            assertNull(p.nextToken());
+            assertEquals(0, p.getParsingContext().getNestingDepth());
+        }
+    }
+
+    @Test
+    public void testSimpleWithEmpty() throws Exception
+    {
+        // 21-Jun-2017, tatu: Depends on setting actually...
+
+        final String XML = "<root><leaf /></root>";
+
+        // -> "{"leaf":null}"
+        try (JsonParser p = _xmlMapper.reader().with(FromXmlParser.Feature.EMPTY_ELEMENT_AS_NULL)
+                .createParser(XML)) {
+            assertTrue(((FromXmlParser) p).isEnabled(FromXmlParser.Feature.EMPTY_ELEMENT_AS_NULL));
+            assertToken(JsonToken.START_OBJECT, p.nextToken());
+            assertToken(JsonToken.FIELD_NAME, p.nextToken());
+            assertEquals("leaf", p.currentName());
+            assertToken(JsonToken.VALUE_NULL, p.nextToken());
+            assertToken(JsonToken.END_OBJECT, p.nextToken());
+            assertNull(p.nextToken());
+        }
+
+        // -> "{"leaf":""}"
+        try (JsonParser p = _xmlMapper.reader().without(FromXmlParser.Feature.EMPTY_ELEMENT_AS_NULL)
+                .createParser(XML)) {
+            assertFalse(((FromXmlParser) p).isEnabled(FromXmlParser.Feature.EMPTY_ELEMENT_AS_NULL));
+            assertToken(JsonToken.START_OBJECT, p.nextToken());
+            assertToken(JsonToken.FIELD_NAME, p.nextToken());
+            assertEquals("leaf", p.currentName());
+            assertToken(JsonToken.VALUE_STRING, p.nextToken());
+            assertEquals("", p.getText());
+            assertToken(JsonToken.END_OBJECT, p.nextToken());
+            assertNull(p.nextToken());
+        }
+    }
+
+    /**
+     * Test that verifies coercion of a "simple" cdata segment within root element
+     * as matching scalar token, similar to how other elements work.
+     */
+    @Test
+    public void testRootScalar() throws Exception
+    {
+        // 02-Jul-2020, tatu: Does not work quite yet
+        final String XML = "<data>value</data>";
+        try (JsonParser p = _xmlMapper.createParser(XML)) {
+            assertToken(JsonToken.START_OBJECT, p.nextToken());
+            assertToken(JsonToken.FIELD_NAME, p.nextToken());
+            assertEquals("", p.currentName());
+            assertToken(JsonToken.VALUE_STRING, p.nextToken());
+            assertEquals("value", p.getText());
+            assertToken(JsonToken.END_OBJECT, p.nextToken());
+            assertNull(p.nextToken());
+            // should be ok to call again tho
+            assertNull(p.nextToken());
+        }
+    }
+
+    @Test
+    public void testRootMixed() throws Exception
+    {
+        // 02-Jul-2020, tatu: Does not work quite yet
+        final String XML = "<data>value<child>abc</child></data>";
+        try (JsonParser p = _xmlMapper.createParser(XML)) {
+            assertToken(JsonToken.START_OBJECT, p.nextToken());
+
+            assertToken(JsonToken.FIELD_NAME, p.nextToken());
+            assertEquals("", p.currentName());
+            assertToken(JsonToken.VALUE_STRING, p.nextToken());
+            assertEquals("value", p.getText());
+
+            assertToken(JsonToken.FIELD_NAME, p.nextToken());
+            assertEquals("child", p.currentName());
+            assertToken(JsonToken.VALUE_STRING, p.nextToken());
+            assertEquals("abc", p.getText());
+
+            assertToken(JsonToken.END_OBJECT, p.nextToken());
+            assertNull(p.nextToken());
+        }
     }
 
     /*
-    /**********************************************************
-    /* Unit tests
-    /**********************************************************
+    /**********************************************************************
+    /* Unit tests, slightly bigger, automated
+    /**********************************************************************
      */
     
-    public void testSimplest() throws Exception
-    {
-        assertEquals("{\"leaf\":\"abc\"}",
-                _readXmlWriteJson("<root><leaf>abc</leaf></root>"));
-    }
-
-    public void testSimpleWithEmpty() throws Exception
-    {
-        // 06-Jan-2015, tatu: Not superbly simple, actually; whether we'll have `null`
-        //    or `{}` depends on context; if in array context, latter; otherwise former
-        assertEquals("{\"leaf\":null}",
-                _readXmlWriteJson("<root><leaf /></root>"));
-    }
-
+    @Test
     public void testSimpleNested() throws Exception
     {
         assertEquals("{\"a\":{\"b\":{\"c\":\"xyz\"}}}",
@@ -58,6 +150,7 @@ public class XmlParserTest extends XmlTestBase
      * specification as XML, and read it back in "as JSON", with
      * expected transformation.
      */
+    @Test
     public void testRoundTripWithSample() throws Exception
     {
         // First: let's convert from sample JSON doc to default xml output
@@ -71,7 +164,7 @@ public class XmlParserTest extends XmlTestBase
         // Former could be worked around; latter less so at this point.
 
         // So, for now, let's just do sort of minimal verification, manually
-        JsonParser p = _xmlMapper.getFactory().createParser(xml);
+        JsonParser p = _xmlMapper.createParser(xml);
         
         assertToken(JsonToken.START_OBJECT, p.nextToken()); // main object
 
@@ -145,6 +238,7 @@ public class XmlParserTest extends XmlTestBase
      * Test to ensure functionality used to force an element to be reported
      * as "JSON" Array, instead of default Object.
      */
+    @Test
     public void testForceElementAsArray() throws Exception
     {
         final String XML = "<array><elem>value</elem><elem><property>123</property></elem><elem>1</elem></array>";
@@ -154,21 +248,21 @@ public class XmlParserTest extends XmlTestBase
         // First: verify handling without forcing array handling:
         assertToken(JsonToken.START_OBJECT, xp.nextToken()); // <array>
         assertToken(JsonToken.FIELD_NAME, xp.nextToken()); // <elem>
-        assertEquals("elem", xp.getCurrentName());
+        assertEquals("elem", xp.currentName());
         assertToken(JsonToken.VALUE_STRING, xp.nextToken());
         assertEquals("value", xp.getText());
 
         assertToken(JsonToken.FIELD_NAME, xp.nextToken()); // <elem>
-        assertEquals("elem", xp.getCurrentName());
+        assertEquals("elem", xp.currentName());
         assertToken(JsonToken.START_OBJECT, xp.nextToken()); // <property>
         assertToken(JsonToken.FIELD_NAME, xp.nextToken());
-        assertEquals("property", xp.getCurrentName());
+        assertEquals("property", xp.currentName());
         assertToken(JsonToken.VALUE_STRING, xp.nextToken());
         assertEquals("123", xp.getText());
         assertToken(JsonToken.END_OBJECT, xp.nextToken()); // <object>
 
         assertToken(JsonToken.FIELD_NAME, xp.nextToken()); // <elem>
-        assertEquals("elem", xp.getCurrentName());
+        assertEquals("elem", xp.currentName());
         assertToken(JsonToken.VALUE_STRING, xp.nextToken());
         assertEquals("1", xp.getText());
 
@@ -176,14 +270,14 @@ public class XmlParserTest extends XmlTestBase
         xp.close();
 
         // And then with array handling:
-        xp = (FromXmlParser) _xmlFactory.createParser(new StringReader(XML));
+        xp = (FromXmlParser) _xmlMapper.createParser(XML);
         assertTrue(xp.getParsingContext().inRoot());
 
         assertToken(JsonToken.START_OBJECT, xp.nextToken()); // <array>
         assertTrue(xp.getParsingContext().inObject()); // true until we do following:
 
         // must request 'as-array' handling, which will "convert" current token:
-        assertTrue("Should 'convert' START_OBJECT to START_ARRAY", xp.isExpectedStartArrayToken());
+        assertTrue(xp.isExpectedStartArrayToken(), "Should 'convert' START_OBJECT to START_ARRAY");
         assertToken(JsonToken.START_ARRAY, xp.getCurrentToken()); // <elem>
         assertTrue(xp.getParsingContext().inArray());
 
@@ -194,7 +288,7 @@ public class XmlParserTest extends XmlTestBase
         assertToken(JsonToken.START_OBJECT, xp.nextToken()); // <property>
         assertTrue(xp.getParsingContext().inObject());
         assertToken(JsonToken.FIELD_NAME, xp.nextToken());
-        assertEquals("property", xp.getCurrentName());
+        assertEquals("property", xp.currentName());
         assertToken(JsonToken.VALUE_STRING, xp.nextToken());
         assertEquals("123", xp.getText());
 
@@ -215,21 +309,22 @@ public class XmlParserTest extends XmlTestBase
         xp.close();
     }
 
+    @Test
     public void testXmlAttributes() throws Exception
     {
         final String XML = "<data max=\"7\" offset=\"9\"/>";
 
-        FromXmlParser xp = (FromXmlParser) _xmlFactory.createParser(new StringReader(XML));
+        FromXmlParser xp = (FromXmlParser) _xmlMapper.createParser(XML);
 
         // First: verify handling without forcing array handling:
         assertToken(JsonToken.START_OBJECT, xp.nextToken()); // <data>
         assertToken(JsonToken.FIELD_NAME, xp.nextToken()); // <max>
-        assertEquals("max", xp.getCurrentName());
+        assertEquals("max", xp.currentName());
         assertToken(JsonToken.VALUE_STRING, xp.nextToken());
         assertEquals("7", xp.getText());
 
         assertToken(JsonToken.FIELD_NAME, xp.nextToken()); // <offset>
-        assertEquals("offset", xp.getCurrentName());
+        assertEquals("offset", xp.currentName());
 
         StringWriter w = new StringWriter();
         assertEquals(6, xp.getText(w));
@@ -246,23 +341,73 @@ public class XmlParserTest extends XmlTestBase
         xp.close();
     }
 
+    @Test
+    public void testMixedContent() throws Exception
+    {
+        String exp = a2q("{'':'first','a':'123','':'second','b':'456','':'last'}");
+        String result = _readXmlWriteJson("<root>first<a>123</a>second<b>456</b>last</root>");
+
+//System.err.println("result = \n"+result);
+        assertEquals(exp, result);
+    }
+
+    @Test
+    public void testInferredNumbers() throws Exception
+    {
+        final String XML = "<data value1='abc' value2='42'>123456789012</data>";
+
+        FromXmlParser xp = (FromXmlParser) _xmlMapper.createParser(XML);
+
+        // First: verify handling without forcing array handling:
+        assertToken(JsonToken.START_OBJECT, xp.nextToken()); // <data>
+        assertToken(JsonToken.FIELD_NAME, xp.nextToken()); // @value1
+        assertEquals("value1", xp.currentName());
+        assertToken(JsonToken.VALUE_STRING, xp.nextToken());
+        assertFalse(xp.isExpectedNumberIntToken());
+        assertEquals("abc", xp.getText());
+
+        assertToken(JsonToken.FIELD_NAME, xp.nextToken()); // @value2
+        assertEquals("value2", xp.currentName());
+        assertToken(JsonToken.VALUE_STRING, xp.nextToken());
+        assertTrue(xp.isExpectedNumberIntToken());
+        assertEquals(JsonToken.VALUE_NUMBER_INT, xp.currentToken());
+        assertEquals(NumberType.INT, xp.getNumberType());
+        assertEquals(42, xp.getIntValue());
+
+        assertToken(JsonToken.FIELD_NAME, xp.nextToken()); // implicit for text
+        assertEquals("", xp.currentName());
+
+        assertToken(JsonToken.VALUE_STRING, xp.nextToken());
+        assertTrue(xp.isExpectedNumberIntToken());
+        assertEquals(JsonToken.VALUE_NUMBER_INT, xp.currentToken());
+        assertEquals(NumberType.LONG, xp.getNumberType());
+        assertEquals(123456789012L, xp.getLongValue());
+        
+        assertToken(JsonToken.END_OBJECT, xp.nextToken()); // </data>
+        xp.close();
+    }
+
     /*
-    /**********************************************************
+    /**********************************************************************
     /* Helper methods
-    /**********************************************************
+    /**********************************************************************
      */
-    
+
     private String _readXmlWriteJson(String xml) throws IOException
     {
-        StringWriter w = new StringWriter();
+        return _readXmlWriteJson(_xmlFactory, xml);
+    }
 
-        JsonParser p = _xmlFactory.createParser(xml);
-        JsonGenerator jg = _jsonFactory.createGenerator(w);
-        while (p.nextToken() != null) {
-            jg.copyCurrentEvent(p);
+    private String _readXmlWriteJson(XmlFactory xmlFactory, String xml) throws IOException
+    {
+        StringWriter w = new StringWriter();
+        try (JsonParser p = xmlFactory.createParser(xml)) {
+            try (JsonGenerator jg = _jsonFactory.createGenerator(w)) {
+                while (p.nextToken() != null) {
+                    jg.copyCurrentEvent(p);
+                }
+            }
         }
-        p.close();
-        jg.close();
         return w.toString();
     }
 }
